@@ -3,28 +3,22 @@ import os
 import sys
 import ctypes
 import gzip
-from typing import TypeVar, Tuple, Optional
+from typing import Tuple, Optional, TYPE_CHECKING
 
-try:
+if TYPE_CHECKING:
     from PIL import Image
-
-    PILLOW_LOADED = True
-except ModuleNotFoundError:
-    PILLOW_LOADED = False
 from .rlottiecommon import LOTLayerNode, LOTMarkerList
 
 # References: rlottie/inc/rlottie.h
 
-TLottieAnimation = TypeVar("TLottieAnimation", bound="LottieAnimation")
-
 
 def frange(start, stop: Optional[float] = None, step: Optional[float] = None):
     # if set start=0.0 and step = 1.0 if not specified
     start = float(start)
-    if stop == None:
+    if stop is None:
         stop = start + 0.0
         start = 0.0
-    if step == None:
+    if step is None:
         step = 1.0
 
     count = 0
@@ -43,8 +37,6 @@ class LottieAnimationPointer(ctypes.c_void_p):
 
 
 class LottieAnimation:
-    rlottie_lib = None
-
     def __init__(
         self,
         path: str = "",
@@ -70,9 +62,6 @@ class LottieAnimation:
             )
 
     def _load_lib(self):
-        if self.rlottie_lib != None:
-            return
-
         if sys.platform.startswith("win32") or sys.platform.startswith("cygwin"):
             rlottie_lib_name = "rlottie.dll"
         elif sys.platform.startswith("darwin"):
@@ -91,10 +80,7 @@ class LottieAnimation:
         else:
             self.rlottie_lib_path = rlottie_lib_name
 
-        if sys.platform.startswith("win32") or sys.platform.startswith("cygwin"):
-            self.rlottie_lib = ctypes.WinDLL(self.rlottie_lib_path)
-        else:
-            self.rlottie_lib = ctypes.CDLL(self.rlottie_lib_path)
+        self.rlottie_lib = ctypes.CDLL(self.rlottie_lib_path)
 
     def __del__(self):
         if self.rlottie_lib:
@@ -108,7 +94,7 @@ class LottieAnimation:
             self.lottie_animation_destroy()
 
     @classmethod
-    def from_file(cls, path: str) -> TLottieAnimation:
+    def from_file(cls, path: str) -> "LottieAnimation":
         """
         Constructs LottieAnimation object from lottie file path.
 
@@ -125,7 +111,7 @@ class LottieAnimation:
         data: str,
         key_size: Optional[int] = None,
         resource_path: Optional[str] = None,
-    ) -> TLottieAnimation:
+    ) -> "LottieAnimation":
         """
         Constructs LottieAnimation object from JSON string data.
 
@@ -141,7 +127,7 @@ class LottieAnimation:
         return cls(data=data, key_size=key_size, resource_path=resource_path)
 
     @classmethod
-    def from_tgs(cls, path: str) -> TLottieAnimation:
+    def from_tgs(cls, path: str) -> "LottieAnimation":
         """
         Constructs LottieAnimation object from tgs file path.
 
@@ -179,8 +165,6 @@ class LottieAnimation:
         self.rlottie_lib.lottie_shutdown.restype = ctypes.c_void_p
         self.rlottie_lib.lottie_shutdown()
 
-        self.rlottie_lib = None
-
     def lottie_animation_from_file(self, path: str):
         """
         Constructs an animation object from JSON file path.
@@ -225,12 +209,12 @@ class LottieAnimation:
         self.data_c = ctypes.create_string_buffer(data.encode())
         data_size = ctypes.sizeof(self.data_c)
 
-        if key_size == None:
+        if key_size is None:
             key_size = data_size
         self.key_c = ctypes.create_string_buffer(key_size)
 
         resource_path_abs = ""
-        if resource_path != None:
+        if resource_path is not None:
             resource_path_abs = os.path.abspath(resource_path)
         self.resource_path_abs_c = ctypes.create_string_buffer(
             resource_path_abs.encode()
@@ -375,7 +359,7 @@ class LottieAnimation:
             LOTLayerNode
         )
 
-        if width == None or height == None:
+        if width is None or height is None:
             width, height = self.lottie_animation_get_size()
 
         render_tree_p = self.rlottie_lib.lottie_animation_render_tree(
@@ -434,13 +418,13 @@ class LottieAnimation:
         :return: rendered surface buffer
         :rtype: bytes
         """
-        if width == None or height == None:
+        if width is None or height is None:
             width, height = self.lottie_animation_get_size()
 
-        if bytes_per_line == None:
+        if bytes_per_line is None:
             bytes_per_line = width * 4
 
-        if buffer_size == None:
+        if buffer_size is None:
             buffer_size = width * height * 4
 
         buffer_c = ctypes.create_string_buffer(buffer_size)
@@ -490,13 +474,13 @@ class LottieAnimation:
         :param Optional[int] height: height of the surface
         :param Optional[int] bytes_per_line: stride of the surface in bytes.
         """
-        if width == None or height == None:
+        if width is None or height is None:
             width, height = self.lottie_animation_get_size()
 
-        if bytes_per_line == None:
+        if bytes_per_line is None:
             bytes_per_line = width * 4
 
-        if buffer_size == None:
+        if buffer_size is None:
             buffer_size = width * height * 4
 
         self.async_buffer_c = ctypes.create_string_buffer(buffer_size)
@@ -547,7 +531,7 @@ class LottieAnimation:
 
         self.rlottie_lib.lottie_animation_render_flush(self.animation_p)
 
-        return self.async_buffer_c
+        return bytes(self.async_buffer_c)
 
     def lottie_animation_property_override(self, _type: str, keypath: str, *args):
         """
@@ -560,9 +544,9 @@ class LottieAnimation:
         lottie_animation_property_override(
         "LOTTIE_ANIMATION_PROPERTY_FILLCOLOR",
         "layer1.group1.fill1",
-        ctypes.float(1.0),
-        ctypes.float(0.0),
-        ctypes.float(0.0)
+        ctypes.c_float(1.0),
+        ctypes.c_float(0.0),
+        ctypes.c_float(0.0)
         )
 
         :param str _type: Property type.
@@ -608,7 +592,10 @@ class LottieAnimation:
             raise IndexError("Invalid _type")
 
         self.rlottie_lib.lottie_animation_property_override(
-            self.animation_p, ctypes.c_int(_type_index), ctypes.c_char_p(keypath), *args
+            self.animation_p,
+            ctypes.c_int(_type_index),
+            ctypes.c_wchar_p(keypath),
+            *args,
         )
 
     def lottie_animation_get_markerlist(self) -> Optional[list]:
@@ -633,7 +620,10 @@ class LottieAnimation:
 
         markerlist = self.rlottie_lib.lottie_animation_get_markerlist(self.animation_p)
 
-        return markerlist.contents
+        try:
+            return markerlist.contents
+        except ValueError:  # NULL pointer access
+            return None
 
     def lottie_configure_model_cache_size(self, cache_size: int):
         """
@@ -655,166 +645,160 @@ class LottieAnimation:
         self.rlottie_lib.lottie_configure_model_cache_size.restype = ctypes.c_void_p
         self.rlottie_lib.lottie_configure_model_cache_size(ctypes.c_size_t(cache_size))
 
-    if PILLOW_LOADED:
+    def render_pillow_frame(
+        self,
+        frame_num: int = 0,
+        buffer_size: Optional[int] = None,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+        bytes_per_line: Optional[int] = None,
+    ) -> "Image.Image":
+        """
+        Create Pillow Image at frame_num
 
-        def render_pillow_frame(
-            self,
-            frame_num: int = 0,
-            buffer_size: Optional[int] = None,
-            width: Optional[int] = None,
-            height: Optional[int] = None,
-            bytes_per_line: Optional[int] = None,
-        ) -> Image.Image:
-            """
-            Create Pillow Image at frame_num
+        :param int frame_num: the frame number needs to be rendered.
+            Defaults to 0.
+        :param Optional[int] buffer_size: size of surface buffer use for rendering
+        :param Optional[int] width: width of the surface
+        :param Optional[int] height: height of the surface
+        :param Optional[int] bytes_per_line: stride of the surface in bytes.
 
-            :param int frame_num: the frame number needs to be rendered.
-                Defaults to 0.
-            :param Optional[int] buffer_size: size of surface buffer use for rendering
-            :param Optional[int] width: width of the surface
-            :param Optional[int] height: height of the surface
-            :param Optional[int] bytes_per_line: stride of the surface in bytes.
+        :return: rendered Pillow Image
+        :rtype: PIL.Image.Image
+        """
+        from PIL import Image
 
-            :return: rendered Pillow Image
-            :rtype: PIL.Image.Image
-            """
-            if width == None or height == None:
-                width, height = self.lottie_animation_get_size()
+        if width is None or height is None:
+            width, height = self.lottie_animation_get_size()
 
-            buffer = self.lottie_animation_render(
-                frame_num=frame_num,
-                buffer_size=buffer_size,
-                width=width,
-                height=height,
-                bytes_per_line=bytes_per_line,
+        buffer = self.lottie_animation_render(
+            frame_num=frame_num,
+            buffer_size=buffer_size,
+            width=width,
+            height=height,
+            bytes_per_line=bytes_per_line,
+        )
+
+        im = Image.frombuffer("RGBA", (width, height), buffer, "raw", "BGRA")
+
+        return im
+
+    def save_frame(
+        self,
+        save_path: str,
+        frame_num: int = 0,
+        buffer_size: Optional[int] = None,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+        bytes_per_line: Optional[int] = None,
+        *args,
+        **kwargs,
+    ):
+        """
+        Save Image at frame_num to save_path
+
+        :param str save_path: path to save the Pillow Image
+        :param int frame_num: the frame number needs to be rendered.
+            Defaults to 0.
+        :param Optional[int] buffer_size: size of surface buffer use for rendering
+        :param Optional[int] width: width of the surface
+        :param Optional[int] height: height of the surface
+        :param Optional[int] bytes_per_line: stride of the surface in bytes.
+        :param *args: additional arguments passing to im.save()
+        :param **kwargs: additional arguments passing to im.save()
+        """
+        im = self.render_pillow_frame(
+            frame_num=frame_num,
+            buffer_size=buffer_size,
+            width=width,
+            height=height,
+            bytes_per_line=bytes_per_line,
+        )
+        im.save(save_path, *args, **kwargs)
+
+    def save_animation(
+        self,
+        save_path: str,
+        fps: Optional[int] = None,
+        frame_num_start: Optional[int] = None,
+        frame_num_end: Optional[int] = None,
+        buffer_size: Optional[int] = None,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+        bytes_per_line: Optional[int] = None,
+        *args,
+        **kwargs,
+    ):
+        """
+        Save Image from frame_num_start to frame_num_end and save it to save_path.
+
+        It is possible to save animation as apng, gif or webp.
+
+        For .gif, maximum framerate is capped at 50.
+
+        Users may override this by specifying fps, at risk of breaking their gif.
+
+        :param str save_path: Path to save the Pillow Image
+        :param Optional[int] fps: Set fps of output image.
+            Will skip frames if lower than original.
+        :param Optional[int] frame_num_start: the starting frame number
+            needs to be rendered.
+        :param Optional[int] frame_num_end: the ending frame number
+            needs to be rendered.
+        :param Optional[int] buffer_size: size of surface buffer use for rendering
+        :param Optional[int] width: width of the surface
+        :param Optional[int] height: height of the surface
+        :param Optional[int] bytes_per_line: stride of the surface in bytes.
+        :param *args: additional arguments passing to im.save()
+        :param **kwargs: additional arguments passing to im.save()
+        """
+        fps_orig = self.lottie_animation_get_framerate()
+        duration = self.lottie_animation_get_duration()
+
+        export_ext = os.path.splitext(save_path)[-1].lower()
+
+        if not fps:
+            fps = fps_orig
+
+            # For .gif, maximum framerate is capped at 50
+            # Users may override this by specifying fps, at risk of breaking their gif
+            # Reference: https://wunkolo.github.io/post/2020/02/buttery-smooth-10fps/
+            if export_ext == ".gif" and fps_orig > 50:
+                fps = 50
+
+        if export_ext == ".gif" and kwargs.get("disposal") is None:
+            kwargs["disposal"] = 2
+
+        if kwargs.get("loop") is None:
+            kwargs["loop"] = 0
+
+        frames = int(duration * fps)
+        frame_duration = 1000 / fps
+
+        if frame_num_start is None:
+            frame_num_start = 0
+        if frame_num_end is None:
+            frame_num_end = frames
+
+        im_list = []
+        for frame in range(frame_num_start, frame_num_end):
+            pos = frame / frame_num_end
+            frame_num = self.lottie_animation_get_frame_at_pos(pos)
+            im_list.append(
+                self.render_pillow_frame(
+                    frame_num=frame_num,
+                    buffer_size=buffer_size,
+                    width=width,
+                    height=height,
+                    bytes_per_line=bytes_per_line,
+                ).copy()
             )
 
-            im = Image.frombuffer("RGBA", (width, height), buffer, "raw", "BGRA")
-
-            return im
-
-        def save_frame(
-            self,
-            save_path: str,
-            frame_num: int = 0,
-            buffer_size: Optional[int] = None,
-            width: Optional[int] = None,
-            height: Optional[int] = None,
-            bytes_per_line: Optional[int] = None,
+        im_list[0].save(
+            save_path,
+            save_all=True,
+            append_images=im_list[1:],
+            duration=int(frame_duration),
             *args,
             **kwargs,
-        ) -> Image.Image:
-            """
-            Save Image at frame_num to save_path
-
-            :param str save_path: path to save the Pillow Image
-            :param int frame_num: the frame number needs to be rendered.
-                Defaults to 0.
-            :param Optional[int] buffer_size: size of surface buffer use for rendering
-            :param Optional[int] width: width of the surface
-            :param Optional[int] height: height of the surface
-            :param Optional[int] bytes_per_line: stride of the surface in bytes.
-            :param *args: additional arguments passing to im.save()
-            :param **kwargs: additional arguments passing to im.save()
-
-            :return: rendered Pillow Image
-            :rtype: PIL.Image.Image
-            """
-            im = self.render_pillow_frame(
-                frame_num=frame_num,
-                buffer_size=buffer_size,
-                width=width,
-                height=height,
-                bytes_per_line=bytes_per_line,
-            )
-            im.save(save_path, *args, **kwargs)
-
-        def save_animation(
-            self,
-            save_path: str,
-            fps: Optional[int] = None,
-            frame_num_start: Optional[int] = None,
-            frame_num_end: Optional[int] = None,
-            buffer_size: Optional[int] = None,
-            width: Optional[int] = None,
-            height: Optional[int] = None,
-            bytes_per_line: Optional[int] = None,
-            *args,
-            **kwargs,
-        ) -> Image.Image:
-            """
-            Save Image from frame_num_start to frame_num_end and save it to save_path.
-
-            It is possible to save animation as apng, gif or webp.
-
-            For .gif, maximum framerate is capped at 50.
-
-            Users may override this by specifying fps, at risk of breaking their gif.
-
-            :param str save_path: Path to save the Pillow Image
-            :param Optional[int] fps: Set fps of output image.
-                Will skip frames if lower than original.
-            :param Optional[int] frame_num_start: the starting frame number
-                needs to be rendered.
-            :param Optional[int] frame_num_end: the ending frame number
-                needs to be rendered.
-            :param Optional[int] buffer_size: size of surface buffer use for rendering
-            :param Optional[int] width: width of the surface
-            :param Optional[int] height: height of the surface
-            :param Optional[int] bytes_per_line: stride of the surface in bytes.
-            :param *args: additional arguments passing to im.save()
-            :param **kwargs: additional arguments passing to im.save()
-
-            :return: rendered Pillow Image
-            :rtype: PIL.Image.Image
-            """
-            fps_orig = self.lottie_animation_get_framerate()
-            duration = self.lottie_animation_get_duration()
-
-            export_ext = os.path.splitext(save_path)[-1].lower()
-
-            if not fps:
-                fps = fps_orig
-
-                # For .gif, maximum framerate is capped at 50
-                # Users may override this by specifying fps, at risk of breaking their gif
-                # Reference: https://wunkolo.github.io/post/2020/02/buttery-smooth-10fps/
-                if export_ext == ".gif" and fps_orig > 50:
-                    fps = 50
-
-            if export_ext == ".gif" and kwargs.get("disposal") == None:
-                kwargs["disposal"] = 2
-
-            if kwargs.get("loop") == None:
-                kwargs["loop"] = 0
-
-            frames = int(duration * fps)
-            frame_duration = 1000 / fps
-
-            if frame_num_start == None:
-                frame_num_start = 0
-            if frame_num_end == None:
-                frame_num_end = frames
-
-            im_list = []
-            for frame in range(frame_num_start, frame_num_end):
-                pos = frame / frame_num_end
-                frame_num = self.lottie_animation_get_frame_at_pos(pos)
-                im_list.append(
-                    self.render_pillow_frame(
-                        frame_num=frame_num,
-                        buffer_size=buffer_size,
-                        width=width,
-                        height=height,
-                        bytes_per_line=bytes_per_line,
-                    ).copy()
-                )
-
-            im_list[0].save(
-                save_path,
-                save_all=True,
-                append_images=im_list[1:],
-                duration=int(frame_duration),
-                *args,
-                **kwargs,
-            )
+        )
