@@ -1,4 +1,14 @@
-from ctypes import *
+#!/usr/bin/env python3
+from ctypes import (
+    c_char,
+    c_char_p,
+    c_size_t,
+    c_float,
+    c_int,
+    c_ubyte,
+    Structure,
+    POINTER,
+)
 
 # References: rlottie/inc/rlottiecommon.h
 
